function main() {
  let xLabels = [
    'Mercedes-AMG',
    'Mercedes-Benz',
    'Porsche',
    'BMW',
    'Audi',
    'Mercedes',
    'Volkswagen',
  ];
  let svg = d3.select('svg'),
    margin = { top: 60, bottom: 60, left: 80, right: 200 },
    width = svg.attr('width') - margin.left - margin.right,
    height =
      svg.attr('height') - margin.top - margin.bottom;
  
  //Add Scales
  let xScale = d3
      .scaleBand()
      .range([margin.left, margin.left + width])
      .padding(0.45),
    yScale = d3
      .scaleLinear()
      .range([height + margin.top, margin.top]),
    colorScale = d3
      .scaleQuantize()
      .range(colorbrewer.Reds[7]);

  //Add Tooltip
  var div = d3
    .select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0);

  //Decimal number format
  var formatDecimal = d3.format(',.2f');

  //Read Data
  d3.csv('barchart.csv', (d) => {
    return {
      code: d['Identification.Make'],
      horsepower: +d[
        'Avg.Engine.Information.Engine.Statistics.Horsepower'
      ],
    };
  }).then((dataset) => {
    let data = [];
    dataset.forEach((elem) => {
      if (xLabels.includes(elem.code)) {
        data.push(elem);
      }
    });
    data.sort((a, b) => a.horsepower - b.horsepower);
    let xSeq = [];
    data.forEach((elem) => xSeq.push(elem.code));

    xScale.domain(xSeq);
    yScale.domain([100, d3.max(data, (d) => d.horsepower)]);
    colorScale.domain([
      100,
      d3.max(data, (d) => d.horsepower),
    ]);

    //Add Graph
    svg
      .selectAll('rect')
      .data(data)
      .enter()
      .append('rect')
      .attr('x', (d) => xScale(d.code))
      .attr('y', (d) => yScale(d.horsepower))
      .attr('width', xScale.bandwidth())
      .attr(
        'height',
        (d) => height + margin.top - yScale(d.horsepower)
      )
      .attr('fill', (d) => colorScale(d.horsepower))
      .on('mouseover', function (event, d) {
        div
          .transition()
          .duration(200)
          .style('opacity', 0.9);
        div
          .html(
            'Engine.Statistics.Horsepower: <br/>' +
              d.code +
              ': ' +
              formatDecimal(d.horsepower)
          )
          .style('left', event.pageX + 'px')
          .style('top', event.pageY - 28 + 'px');
      })
      .on('mouseout', function (d) {
        div.transition().duration(500).style('opacity', 0);
      });

    //Add Title
    svg
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('x', width / 2 + margin.left)
      .attr('y', margin.top / 2)
      .text('Bar chart with a Sequential color scale');

    //X Axis
    svg
      .append('g')
      .attr('class', 'axis')
      .attr(
        'transform',
        'translate(0, ' + (height + margin.top) + ')'
      )
      .call(d3.axisBottom(xScale).tickValues(xLabels))
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('x', width / 2 + margin.left)
      .attr('y', margin.bottom / 2)     
      .text('Identification.Make');

    //Y Axis
    svg
      .append('g')
      .attr('class', 'axis')
      .attr(
        'transform',
        'translate(' + margin.left + ', 0)'
      )
      .call(
        d3
          .axisLeft(yScale)
          .ticks(5)
          .tickFormat(d3.format('~s'))
      )
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('transform', 'rotate(-90)')
      .attr('x', -height / 2 - margin.top)
      .attr('y', -margin.left / 2)     
      .text(
        'Avg.Engine.Information.Engine.Statistics.Horsepower'
      );

    //Add Legend
    let legend = d3
      .legendColor()
      .labelFormat(d3.format('.2s'))
      .title('Bar chart Legend')
      .scale(colorScale);
    svg
      .append('g')
      .attr('class', 'legend')
      .attr(
        'transform',
        'translate(' +
          (width + margin.left + margin.right / 5) +
          ',' +
          (height - margin.bottom) / 2 +
          ')'
      )
      .call(legend);
  });
}

main();
